#! /usr/bin/python

#Fabio Eboli 16/Jan 2012

from struct import *
import os
import sys

# class owonfile
# methods
# FileIdentify
#   checks the dso signature and number of traces in file
#   if the file is readable self.blocks contains unoe block per trace,
#   that carries trace informations
# PrintInfo
#   prints basic info about the file and the blocks found

# class dsotrace
# methods
# LoadTraceDataFromFile
#   loads trace data from specified file and trace number
# LoadTraceInfoParFile
#   loads trace parameters from specified file and trace number
# PrintTraceInfo
#   prints informations about specified trace previously loaded
# Convert
#   transforms the raw data loaded from LoadTraceDataFromFile
#   in two lists of Time-Voltage values according to parameters
#   loaded from LoadTraceParFromFile

# bin file format: (big endian)
# meaning, size, type, abs.offset, rel.offset to channel string
# preamble:
#   model signature string, 6 bytes, string, 0x00,-0x0A
#   unknown, int, 4bytes, 0x06, -0x04
# first trace:
#   channel string, 3 bytes, string, 0x0A, 0x00
#   trace data size, 4 bytes, int,0x0D, 0x03
#   unknown, 4 bytes, int, 0x11, 0x07
#   First Point to screen, 4 bytes, int, 0x15, 0x0B
#   Number of point to screen, 4 bytes, int, 0x19, 0x0F
#   Memory depth, 4 bytes, int, 0x1D, 0x13
#   slow scan, 4 bytes, int, 0x21, 0x17
#   T/div, 4 bytes, int, 0x25, 0x1B
#   Zero, 4 bytes, int, 0x29, 0x1F
#   V/Div, 4 bytes, int, 0x2D, 0x23
#   Probe attn, 4 bytes, int, 0x31, 0x27
#   unknown, 4 bytes, float, 0x35, 0x2B
#   frequency, 4 bytes, float, 0x39, 0x2F
#   period, 4 bytes, float, 0x3D, 0x33
#   mV per bit, 4 bytes, float, 0x41, 0x37
#   data,1 byte*Memory depth,0x45,0x3B
# second trace (eventual):
#   ... like first trace, change only abs. offsets

class owonfile:

    def __init__(self):
        self.name=""
        self.blocks=[]
        self.DsoStr=""
        self.Seq=[1,2,5,10,20,50,100,200,500]
        self.SeqT=['nS','uS','mS','S']
        self.SeqTs=[0.000000001,0.000001,0.001,1]
        self.SeqV=['mV','V']
        self.SeqVv=[0.001,1]
        self.ProbeAttn=[1,10,100,1000]

    def FileIdentify(self,filename):
        self.name=filename
        self.blocks=[]
        f=open(filename,'rb')
        self.DsoStr=f.read(6)
        if self.DsoStr=="SPBS02": #Check dso model signature
            BlockNumber=0
            BlockStart=0x0A   #first "CHx" string
            f.seek(BlockStart)
            CHStr=f.read(2)
            while CHStr=="CH":# till the EOF
                BlockNumber=BlockNumber+1
                f.seek(BlockStart)
                Channel=f.read(3)
                f.seek(0x03+BlockStart)
                BlockSize=unpack('i',f.read(4))[0] #block size 
                if BlockSize<0:
                    BlockSize=-BlockSize
                BlockSize=BlockSize+3  #including "CHx"
                f.seek(0x13+BlockStart)
                NumSamples=unpack('i',f.read(4))[0] # Memory depth
                f.seek(0x1B+BlockStart)
                Tdiv=unpack('i',f.read(4))[0]       # Time Per division
                TimePerDivStr=str(self.Seq[(Tdiv+1)%9])+self.SeqT[(Tdiv+1)/9]+"/div"
                f.seek(0x23+BlockStart)
                Vdiv=unpack('i',f.read(4))[0]       # Voltage per division
                VoltPerDivStr=str(self.Seq[(Vdiv+1)%9])+self.SeqV[(Vdiv+1)/9]+"/div"
                f.seek(0x2F+BlockStart)
                Freq=unpack('f',f.read(4))[0]        # frequency
                f.seek(0x33+BlockStart)
                Period=unpack('f',f.read(4))[0]      # period
                self.blocks.append([BlockNumber,BlockStart,BlockSize,Channel,NumSamples,TimePerDivStr,VoltPerDivStr,Freq,Period])
                BlockStart=BlockStart+BlockSize
                f.seek(BlockStart)
                CHStr=f.read(2)
            f.close()
        else:
            f.close()

    def PrintInfo(self):
        print('-------Parameters-----------')
        print("Device code: "+self.DsoStr)
        print("Found "+str(len(self.blocks))+" blocks")
        for i in range(len(self.blocks)):
            print("---------Block "+str(self.blocks[i][0])+"----------")
            print("Block start: "+str(self.blocks[i][1]))
            print("Block Lenght: "+str(self.blocks[i][2]))
            print("Channel: "+self.blocks[i][3])
            print("Samples number: "+str(self.blocks[i][4]))
            print("Time per division: "+str(self.blocks[i][5]))
            print("Volt per division: "+str(self.blocks[i][6]))
            print("Frequency: "+str(self.blocks[i][7]))
            print("Period: "+str(self.blocks[i][8]))
            

class dsotrace:

    def __init__(self):
        self.filename=""
        self.BlockStart=0
        self.Channel=""
        self.DataBlockLenght=0
        self.TimePerDivStr=""
        self.TimePerDiv=0.0
        self.NumSamples=0
        self.SperSample=0
        self.TimeSamples=[]
        self.SamplesRaw=[]
        self.ZeroLev=0
        self.Samples=[]
        self.Freq=0
        self.Period=0
        self.mVperBit=0
        self.ClipLevelH=0
        self.ClipLevelL=0
        self.ProbeAttn=0
        self.VoltPerDivStr=""
        self.VoltPerDiv=0
        self.SlowSamp=0
        self.UnknA=0
        self.UnknC=0
        self.UnknD=0
        self.UnknE=0
        self.UnknF=0
        self.Seq=[1,2,5,10,20,50,100,200,500]
        self.SeqT=['nS','uS','mS','S']
        self.SeqTs=[0.000000001,0.000001,0.001,1]
        self.SeqV=['mV','V']
        self.SeqVv=[0.001,1]
        self.PrAttn=[1,10,100,1000]

    #loads trace data points from the file
    def LoadTraceDataFromFile(self,filename,trace):
        filetoload=owonfile()
        filetoload.FileIdentify(filename)
        self.filename=filename
        self.BlockStart=filetoload.blocks[trace][1]
        self.SamplesRaw=[]
        f=open(filename,'rb')
        f.seek(0x3B+self.BlockStart)
        for i in range(self.NumSamples) :
            self.SamplesRaw.append(unpack('b',f.read(1))[0])
            if i%1000==0 :
                percent_completed=int(i*100/self.NumSamples)
                sys.stdout.write("\r read %d%% samples from file" % percent_completed)
                sys.stdout.flush()
        f.close()

    #Loads paramteres of the trace from the file
    def LoadTraceParFromFile(self,filename,trace):
        filetoload=owonfile()
        filetoload.FileIdentify(filename)
        self.filename=filename
        self.BlockStart=filetoload.blocks[trace][1]
        self.SamplesRaw=[]
        f=open(filename,'rb')
        f.seek(self.BlockStart)
        self.Channel=f.read(3)
        self.DataBlockLenght=unpack('i',f.read(4))[0]     # data lenght
        self.UnknC=unpack('i',f.read(4))[0]     # unknown
        self.FirstPTS=unpack('i',f.read(4))[0]     # first point to screen
        self.NumberPTS=unpack('i',f.read(4))[0]     # number of points to screen
        self.NumSamples=unpack('i',f.read(4))[0]    # memory lenght
        self.SlowSamp=unpack('i',f.read(4))[0]  # slow scanning
        Tdiv=unpack('i',f.read(4))[0]       # time per division
        self.TimePerDivStr=str(self.Seq[(Tdiv+1)%9])+self.SeqT[(Tdiv+1)/9]
        self.TimePerDiv=(self.Seq[(Tdiv+1)%9])*(self.SeqTs[(Tdiv+1)/9]) #numeric format
        self.ZeroLev=unpack('i',f.read(4))[0]        # zero voltage
        Vdiv=unpack('i',f.read(4))[0]       # volt per division
        self.VoltPerDivStr=str(self.Seq[(Vdiv+1)%9])+self.SeqV[(Vdiv+1)/9]
        self.VoltPerDiv=(self.Seq[(Vdiv+1)%9])*(self.SeqVv[(Vdiv+1)/9])
        self.ProbeAttn=self.PrAttn[unpack('i',f.read(4))[0]]  # probe attenuation
        self.UnknF=unpack('f',f.read(4))[0]     # unknown
        self.Freq=unpack('f',f.read(4))[0]       # frequency
        self.Period=unpack('f',f.read(4))[0]     # period
        self.mVperBit=unpack('f',f.read(4))[0]       # Voltage multipier mV per LSB
        self.SperSample=self.TimePerDiv/self.NumSamples*20
        f.close()

    #Prints the trace info
    def PrintTraceInfo(self,trace):
        print('-------Trace Info-----------')
        print("Channel: "+self.Channel)
        print("Block Lenght: "+str(self.DataBlockLenght))
        print("unknc : "+str(self.UnknC))
        print("First point to screen: "+str(self.FirstPTS))
        print("Number of points to screen "+str(self.NumberPTS))
        print("Number of samples: "+str(self.NumSamples))
        print("Slow scan: "+str(self.SlowSamp))
        print("Time per division: "+ self.TimePerDivStr)
        print("Zero volt: "+str(self.ZeroLev))
        print("Volt per division: "+self.VoltPerDivStr)
        print("Probe Attenuation: "+str(self.ProbeAttn))
        print("unknown: "+str(self.UnknF))
        print("Frequency: "+str(self.Freq))
        print("Period: "+str(self.Period))
        print("Voltage scaling mV per bit: "+str(self.mVperBit))
        print("Seconds per sample: "+str(self.SperSample))

    #Data conversion
    def Convert(self):
        self.Samples=[]
        self.TimeSamples=[]
        for i in range(len(self.SamplesRaw)) :
            self.ClipLevelH=(127-self.ZeroLev)*self.mVperBit/1000*self.ProbeAttn
            self.ClipLevelL=(-128-self.ZeroLev)*self.mVperBit/1000*self.ProbeAttn
            self.Samples.append(((self.SamplesRaw[i])-self.ZeroLev)*self.mVperBit/1000*self.ProbeAttn)
            self.TimeSamples.append(self.SperSample*i)


